# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2014 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""ml2 binding:vif_details

Revision ID: 50d5ba354c23
Revises: 27cc183af192
Create Date: 2014-02-11 23:21:59.577972

"""

# revision identifiers, used by Alembic.
revision = '50d5ba354c23'
down_revision = '27cc183af192'

# Change to ['*'] if this migration applies to all plugins

migration_for_plugins = [
    'neutron.plugins.ml2.plugin.Ml2Plugin'
]

from alembic import context
from alembic import op
import sqlalchemy as sa

from neutron.db import migration


def upgrade(active_plugins=None, options=None):
    if not migration.should_run(active_plugins, migration_for_plugins):
        return

    bind = op.get_bind()
    engine = bind.engine
    if engine.name == 'sqlite':
        op.execute("CREATE TEMPORARY TABLE ml2_port_bindings_backup ( "
                   "port_id VARCHAR(36) NOT NULL, "
                   "host VARCHAR(255) NOT NULL, "
                   "vif_type VARCHAR(64) NOT NULL, "
                   "cap_port_filter BOOLEAN NOT NULL, "
                   "driver VARCHAR(64), "
                   "segment VARCHAR(36), "
                   "vnic_type VARCHAR(64) DEFAULT 'normal' NOT NULL, "
                   "vif_details VARCHAR(4095) DEFAULT '' NOT NULL, "
                   "PRIMARY KEY (port_id), "
                   "FOREIGN KEY(port_id) REFERENCES ports (id) ON DELETE CASCADE, "
                   "FOREIGN KEY(segment) REFERENCES ml2_network_segments (id) ON DELETE SET NULL, "
                   "CHECK (cap_port_filter IN (0, 1)));")
        op.execute("INSERT INTO ml2_port_bindings_backup "
                   "(port_id,host,vif_type,cap_port_filter,driver,segment,vnic_type) "
                   "SELECT port_id,host,vif_type,cap_port_filter,driver,segment,vnic_type "
                   "FROM ml2_port_bindings;")
        for value in ('true', 'false'):
            op.execute("UPDATE ml2_port_bindings_backup SET"
                       " vif_details = '{\"port_filter\": %(value)s}'"
                       " WHERE cap_port_filter = '%(value)s'" % {'value': value})
        op.execute("DROP TABLE ml2_port_bindings")
        op.execute("CREATE TABLE ml2_port_bindings ( "
                   "port_id VARCHAR(36) NOT NULL, "
                   "host VARCHAR(255) NOT NULL, "
                   "vif_type VARCHAR(64) NOT NULL, "
                   "driver VARCHAR(64), "
                   "segment VARCHAR(36), "
                   "vnic_type VARCHAR(64) DEFAULT 'normal' NOT NULL, "
                   "vif_details VARCHAR(4095) DEFAULT '' NOT NULL, "
                   "PRIMARY KEY (port_id), "
                   "FOREIGN KEY(port_id) REFERENCES ports (id) ON DELETE CASCADE, "
                   "FOREIGN KEY(segment) REFERENCES ml2_network_segments (id) ON DELETE SET NULL);")
        op.execute("INSERT INTO ml2_port_bindings "
                   "SELECT port_id,host,vif_type,driver,segment,vnic_type,vif_details "
                   "FROM ml2_port_bindings_backup;")
        op.execute("DROP TABLE ml2_port_bindings_backup")
    else:
        op.add_column('ml2_port_bindings',
                      sa.Column('vif_details', sa.String(length=4095),
                                nullable=False, server_default=''))
        migr_context = context.get_context()
        with context.begin_transaction():
            for value in ('true', 'false'):
                migr_context.execute(
                    "UPDATE ml2_port_bindings SET"
                    " vif_details = '{\"port_filter\": %(value)s}'"
                    " WHERE cap_port_filter = %(value)s" % {'value': value})
        op.drop_column('ml2_port_bindings', 'cap_port_filter')


def downgrade(active_plugins=None, options=None):
    if not migration.should_run(active_plugins, migration_for_plugins):
        return

    op.add_column('ml2_port_bindings',
                  sa.Column('cap_port_filter', sa.Boolean(),
                            nullable=False, default=False))
    migr_context = context.get_context()
    with context.begin_transaction():
        migr_context.execute(
            "UPDATE ml2_port_bindings SET"
            " cap_port_filter = true"
            " WHERE vif_details LIKE '%\"port_filter\": true%'")
    op.drop_column('ml2_port_bindings', 'vif_details')
